@@ -12,8 +12,7 @@ use crate::error::ErrorContext as EC;
 use crate::lang::SgLang;
 use crate::print::{ColoredPrinter, Diff, Heading, InteractivePrinter, JSONPrinter, Printer};
 use crate::utils::{filter_file_pattern, InputArgs, MatchUnit, OutputArgs};
-use crate::utils::{run_std_in, StdInWorker};
-use crate::utils::{run_worker, Items, Worker};
+use crate::utils::{Items, PathWorker, StdInWorker, Worker};
 
 // NOTE: have to register custom lang before clap read arg
 // RunArg has a field of SgLang
@@ -123,7 +122,7 @@ pub fn run_with_pattern(arg: RunArg) -> Result<()> {
     .context(context);
   let interactive = arg.output.needs_interactive();
   if interactive {
-    let from_stdin = arg.input.is_stdin();
+    let from_stdin = arg.input.stdin;
     let printer = InteractivePrinter::new(printer, arg.output.update_all, from_stdin)?;
     run_pattern_with_printer(arg, printer)
   } else {
@@ -132,12 +131,12 @@ pub fn run_with_pattern(arg: RunArg) -> Result<()> {
 }
 
 fn run_pattern_with_printer(arg: RunArg, printer: impl Printer + Sync) -> Result<()> {
-  if arg.input.is_stdin() {
-    run_std_in(RunWithSpecificLang::new(arg, printer)?)
+  if arg.input.stdin {
+    RunWithSpecificLang::new(arg, printer)?.run_std_in()
   } else if arg.lang.is_some() {
-    run_worker(RunWithSpecificLang::new(arg, printer)?)
+    RunWithSpecificLang::new(arg, printer)?.run_path()
   } else {
-    run_worker(RunWithInferredLang { arg, printer })
+    RunWithInferredLang { arg, printer }.run_path()
   }
 }
 
@@ -145,19 +144,8 @@ struct RunWithInferredLang<Printer> {
   arg: RunArg,
   printer: Printer,
 }
-
 impl<P: Printer + Sync> Worker for RunWithInferredLang<P> {
   type Item = (MatchUnit<Pattern<SgLang>>, SgLang);
-  fn build_walk(&self) -> WalkParallel {
-    self.arg.input.walk()
-  }
-
-  fn produce_item(&self, path: &Path) -> Option<Self::Item> {
-    let lang = SgLang::from_path(path)?;
-    let matcher = Pattern::try_new(&self.arg.pattern, lang).ok()?;
-    let match_unit = filter_file_pattern(path, lang, matcher)?;
-    Some((match_unit, lang))
-  }
 
   fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
     let rewrite = &self.arg.rewrite;
@@ -182,6 +170,19 @@ impl<P: Printer + Sync> Worker for RunWithInferredLang<P> {
   }
 }
 
+impl<P: Printer + Sync> PathWorker for RunWithInferredLang<P> {
+  fn build_walk(&self) -> WalkParallel {
+    self.arg.input.walk()
+  }
+
+  fn produce_item(&self, path: &Path) -> Option<Self::Item> {
+    let lang = SgLang::from_path(path)?;
+    let matcher = Pattern::try_new(&self.arg.pattern, lang).ok()?;
+    let match_unit = filter_file_pattern(path, lang, matcher)?;
+    Some((match_unit, lang))
+  }
+}
+
 struct RunWithSpecificLang<Printer> {
   arg: RunArg,
   printer: Printer,
@@ -203,16 +204,7 @@ impl<Printer> RunWithSpecificLang<Printer> {
 
 impl<P: Printer + Sync> Worker for RunWithSpecificLang<P> {
   type Item = MatchUnit<Pattern<SgLang>>;
-  fn build_walk(&self) -> WalkParallel {
-    let lang = self.arg.lang.expect("must present");
-    self.arg.input.walk_lang(lang)
-  }
-  fn produce_item(&self, path: &Path) -> Option<Self::Item> {
-    let arg = &self.arg;
-    let pattern = self.pattern.clone();
-    let lang = arg.lang.expect("must present");
-    filter_file_pattern(path, lang, pattern)
-  }
+
   fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
     let printer = &self.printer;
     printer.before_print()?;
@@ -240,6 +232,19 @@ impl<P: Printer + Sync> Worker for RunWithSpecificLang<P> {
   }
 }
 
+impl<P: Printer + Sync> PathWorker for RunWithSpecificLang<P> {
+  fn build_walk(&self) -> WalkParallel {
+    let lang = self.arg.lang.expect("must present");
+    self.arg.input.walk_lang(lang)
+  }
+  fn produce_item(&self, path: &Path) -> Option<Self::Item> {
+    let arg = &self.arg;
+    let pattern = self.pattern.clone();
+    let lang = arg.lang.expect("must present");
+    filter_file_pattern(path, lang, pattern)
+  }
+}
+
 impl<P: Printer + Sync> StdInWorker for RunWithSpecificLang<P> {
   fn parse_stdin(&self, src: String) -> Option<Self::Item> {
     let lang = self.arg.lang.expect("must present");
