@@ -1,62 +1,27 @@
 #![cfg(not(feature = "napi-noop-in-unit-test"))]
 
 mod doc;
+mod fe_lang;
 mod sg_node;
 
-use ast_grep_config::{RuleWithConstraint, SerializableRuleCore};
+use ast_grep_config::RuleWithConstraint;
 use ast_grep_core::language::Language;
 use ast_grep_core::pinned::{NodeData, PinnedNodeData};
 use ast_grep_core::{AstGrep, NodeMatch};
-use ignore::types::TypesBuilder;
 use ignore::{WalkBuilder, WalkParallel, WalkState};
-use napi::anyhow::{anyhow, Context, Error, Result as Ret};
+use napi::anyhow::{anyhow, Context, Result as Ret};
 use napi::bindgen_prelude::*;
 use napi::threadsafe_function::{ErrorStrategy, ThreadsafeFunction, ThreadsafeFunctionCallMode};
 use napi::{JsNumber, Task};
 use napi_derive::napi;
+use std::collections::HashMap;
 use std::sync::atomic::{AtomicU32, Ordering};
 use std::sync::mpsc::channel;
 
-use doc::{FrontEndLanguage, JsDoc};
+use doc::{JsDoc, NapiConfig};
+use fe_lang::{build_files, FrontEndLanguage, LangOption};
 use sg_node::{SgNode, SgRoot};
 
-/// Rule configuration similar to YAML
-/// See https://ast-grep.github.io/reference/yaml.html
-#[napi(object)]
-pub struct NapiConfig {
-  /// The rule object, see https://ast-grep.github.io/reference/rule.html
-  pub rule: serde_json::Value,
-  /// See https://ast-grep.github.io/guide/rule-config.html#constraints
-  pub constraints: Option<serde_json::Value>,
-  /// Available languages: html, css, js, jsx, ts, tsx
-  pub language: Option<FrontEndLanguage>,
-  /// https://ast-grep.github.io/reference/yaml.html#transform
-  pub transform: Option<serde_json::Value>,
-  /// https://ast-grep.github.io/guide/rule-config/utility-rule.html
-  pub utils: Option<serde_json::Value>,
-}
-
-fn parse_config(
-  config: NapiConfig,
-  language: FrontEndLanguage,
-) -> Result<RuleWithConstraint<FrontEndLanguage>> {
-  let lang = config.language.unwrap_or(language);
-  let rule = SerializableRuleCore {
-    language: lang,
-    rule: serde_json::from_value(config.rule)?,
-    constraints: config.constraints.map(serde_json::from_value).transpose()?,
-    transform: config.transform.map(serde_json::from_value).transpose()?,
-    utils: config.utils.map(serde_json::from_value).transpose()?,
-  };
-  rule.get_matcher(&Default::default()).map_err(|e| {
-    let error = Error::from(e)
-      .chain()
-      .map(ToString::to_string)
-      .collect::<Vec<_>>();
-    napi::Error::new(napi::Status::InvalidArg, error.join("\n |->"))
-  })
-}
-
 macro_rules! impl_lang_mod {
     ($name: ident, $lang: ident) =>  {
       #[napi]
@@ -122,26 +87,6 @@ impl_lang_mod!(ts, TypeScript);
 impl_lang_mod!(tsx, Tsx);
 impl_lang_mod!(css, Css);
 
-fn build_files(paths: Vec<String>) -> Result<WalkParallel> {
-  if paths.is_empty() {
-    return Err(anyhow!("paths cannot be empty.").into());
-  }
-  let types = TypesBuilder::new()
-    .add_defaults()
-    .select("css")
-    .select("html")
-    .select("js")
-    .select("ts")
-    .build()
-    .unwrap();
-  let mut paths = paths.into_iter();
-  let mut builder = WalkBuilder::new(paths.next().unwrap());
-  for path in paths {
-    builder.add(path);
-  }
-  let walk = builder.types(types).build_parallel();
-  Ok(walk)
-}
 pub struct ParseAsync {
   src: String,
   lang: FrontEndLanguage,
@@ -161,10 +106,13 @@ impl Task for ParseAsync {
   }
 }
 
+type Entry = std::result::Result<ignore::DirEntry, ignore::Error>;
+
 pub struct IterateFiles<D> {
   walk: WalkParallel,
+  lang_option: LangOption,
   tsfn: D,
-  producer: fn(&D, std::result::Result<ignore::DirEntry, ignore::Error>) -> Ret<bool>,
+  producer: fn(&D, Entry, &LangOption) -> Ret<bool>,
 }
 
 impl<T: 'static + Send + Sync> Task for IterateFiles<T> {
@@ -180,7 +128,8 @@ impl<T: 'static + Send + Sync> Task for IterateFiles<T> {
     walker.run(|| {
       let tx = tx.clone();
       let file_count = &file_count;
-      Box::new(move |entry| match producer(tsfn, entry) {
+      let lang_option = &self.lang_option;
+      Box::new(move |entry| match producer(tsfn, entry, lang_option) {
         Ok(true) => {
           // file is sent to JS thread, increment file count
           if tx.send(()).is_ok() {
@@ -214,17 +163,31 @@ const THREAD_FUNC_QUEUE_SIZE: usize = 1000;
 
 type ParseFiles = IterateFiles<ThreadsafeFunction<SgRoot, ErrorStrategy::CalleeHandled>>;
 
-#[napi(
-  ts_args_type = "paths: string[], callback: (err: null | Error, result: SgRoot) => void",
-  ts_return_type = "Promise<number>"
-)]
-pub fn parse_files(paths: Vec<String>, callback: JsFunction) -> Result<AsyncTask<ParseFiles>> {
+#[napi(object)]
+pub struct FileOption {
+  pub paths: Vec<String>,
+  pub language_globs: HashMap<String, Vec<String>>,
+}
+
+#[napi(ts_return_type = "Promise<number>")]
+pub fn parse_files(
+  paths: Either<Vec<String>, FileOption>,
+  #[napi(ts_arg_type = "(err: null | Error, result: SgRoot) => void")] callback: JsFunction,
+) -> Result<AsyncTask<ParseFiles>> {
   let tsfn: ThreadsafeFunction<SgRoot, ErrorStrategy::CalleeHandled> =
     callback.create_threadsafe_function(THREAD_FUNC_QUEUE_SIZE, |ctx| Ok(vec![ctx.value]))?;
-  let walk = build_files(paths)?;
+  let (paths, globs) = match paths {
+    Either::A(v) => (v, HashMap::new()),
+    Either::B(FileOption {
+      paths,
+      language_globs,
+    }) => (paths, FrontEndLanguage::lang_globs(language_globs)),
+  };
+  let walk = build_files(paths, &globs)?;
   Ok(AsyncTask::new(ParseFiles {
     walk,
     tsfn,
+    lang_option: LangOption::infer(&globs),
     producer: call_sg_root,
   }))
 }
@@ -233,6 +196,7 @@ pub fn parse_files(paths: Vec<String>, callback: JsFunction) -> Result<AsyncTask
 fn call_sg_root(
   tsfn: &ThreadsafeFunction<SgRoot, ErrorStrategy::CalleeHandled>,
   entry: std::result::Result<ignore::DirEntry, ignore::Error>,
+  lang_option: &LangOption,
 ) -> Ret<bool> {
   let entry = entry?;
   if !entry
@@ -242,29 +206,18 @@ fn call_sg_root(
   {
     return Ok(false);
   }
-  let (root, path) = get_root(entry)?;
+  let (root, path) = get_root(entry, lang_option)?;
   let sg = SgRoot(root, path);
   tsfn.call(Ok(sg), ThreadsafeFunctionCallMode::Blocking);
   Ok(true)
 }
 
-fn get_root(entry: ignore::DirEntry) -> Ret<(AstGrep<JsDoc>, String)> {
-  use FrontEndLanguage::*;
+fn get_root(entry: ignore::DirEntry, lang_option: &LangOption) -> Ret<(AstGrep<JsDoc>, String)> {
   let path = entry.into_path();
   let file_content = std::fs::read_to_string(&path)?;
-  let ext = path
-    .extension()
-    .context("check file")?
-    .to_str()
-    .context("to str")?;
-  let lang = match ext {
-    "css" | "scss" => Css,
-    "html" | "htm" | "xhtml" => Html,
-    "cjs" | "js" | "mjs" | "jsx" | "mjsx" | "cjsx" => JavaScript,
-    "ts" | "mts" | "cts" => TypeScript,
-    "tsx" | "mtsx" | "ctsx" => Tsx,
-    _ => return Err(anyhow!("file not recognized")),
-  };
+  let lang = lang_option
+    .get_lang(&path)
+    .context(anyhow!("file not recognized"))?;
   let doc = JsDoc::new(file_content, lang);
   Ok((AstGrep::doc(doc), path.to_string_lossy().into()))
 }
@@ -287,44 +240,10 @@ pub struct FindConfig {
   pub paths: Vec<String>,
   /// a Rule object to find what nodes will match
   pub matcher: NapiConfig,
-}
-
-fn select_custom<'b>(
-  builder: &'b mut TypesBuilder,
-  file_type: &str,
-  suffix_list: &[&str],
-) -> &'b mut TypesBuilder {
-  for suffix in suffix_list {
-    builder
-      .add(file_type, suffix)
-      .expect("file pattern must compile");
-  }
-  builder.select(file_type)
-}
-
-fn find_files_with_lang(paths: Vec<String>, lang: &FrontEndLanguage) -> Result<WalkParallel> {
-  if paths.is_empty() {
-    return Err(anyhow!("paths cannot be empty.").into());
-  }
-
-  let mut types = TypesBuilder::new();
-  let types = types.add_defaults();
-  let types = match lang {
-    FrontEndLanguage::TypeScript => select_custom(types, "myts", &["*.ts", "*.mts", "*.cts"]),
-    FrontEndLanguage::Tsx => select_custom(types, "mytsx", &["*.tsx", "*.mtsx", "*.ctsx"]),
-    FrontEndLanguage::Css => types.select("css"),
-    FrontEndLanguage::Html => types.select("html"),
-    FrontEndLanguage::JavaScript => types.select("js"),
-  }
-  .build()
-  .unwrap();
-  let mut paths = paths.into_iter();
-  let mut builder = WalkBuilder::new(paths.next().unwrap());
-  for path in paths {
-    builder.add(path);
-  }
-  let walk = builder.types(types).build_parallel();
-  Ok(walk)
+  /// An list of pattern globs to treat of certain files in the specified language.
+  /// eg. ['*.vue', '*.svelte'] for html.findFiles, or ['*.ts'] for tsx.findFiles.
+  /// It is slightly different from https://ast-grep.github.io/reference/sgconfig.html#languageglobs
+  pub language_globs: Option<Vec<String>>,
 }
 
 fn find_in_files_impl(
@@ -335,11 +254,17 @@ fn find_in_files_impl(
   let tsfn = callback.create_threadsafe_function(THREAD_FUNC_QUEUE_SIZE, |ctx| {
     from_pinned_data(ctx.value, ctx.env)
   })?;
-  let rule = parse_config(config.matcher, lang)?;
-  let walk = find_files_with_lang(config.paths, &lang)?;
+  let FindConfig {
+    paths,
+    matcher,
+    language_globs,
+  } = config;
+  let rule = matcher.parse_with(lang)?;
+  let walk = lang.find_files(paths, language_globs)?;
   Ok(AsyncTask::new(FindInFiles {
     walk,
     tsfn: (tsfn, rule),
+    lang_option: LangOption::Specified(lang),
     producer: call_sg_node,
   }))
 }
@@ -370,6 +295,7 @@ fn call_sg_node(
     RuleWithConstraint<FrontEndLanguage>,
   ),
   entry: std::result::Result<ignore::DirEntry, ignore::Error>,
+  lang_option: &LangOption,
 ) -> Ret<bool> {
   let entry = entry?;
   if !entry
@@ -379,7 +305,7 @@ fn call_sg_node(
   {
     return Ok(false);
   }
-  let (root, path) = get_root(entry)?;
+  let (root, path) = get_root(entry, lang_option)?;
   let mut pinned = PinnedNodeData::new(root.inner, |r| r.root().find_all(rule).collect());
   let hits: &Vec<_> = pinned.get_data();
   if hits.is_empty() {
